
(module talk (lib "slideshow-run.ss" "texpict")
  (require (lib "mred.ss" "mred")
	   (lib "class.ss")
	   (lib "math.ss")
	   (lib "list.ss")
	   "mkns.ss")

  (set-use-background-frame! #t)
  (define demo-inset (make-slide-inset 0 0 0 200))

  (define subtalk-custodian #f)
  (define subtalk-eventspace #f)
  (define subtalk-namespace #f)

  (define (end-subtalk)
    (when subtalk-custodian
      (custodian-shutdown-all subtalk-custodian))
    (set! subtalk-custodian #f)
    (set! subtalk-eventspace #f)
    (set! subtalk-namespace #f))

  (define (begin-subtalk)
    (unless subtalk-custodian
      (set! subtalk-custodian (make-custodian))
      (parameterize ([current-custodian subtalk-custodian])
	(set! subtalk-eventspace (make-eventspace))
	(parameterize ([current-eventspace subtalk-eventspace])
	  (set! subtalk-namespace (v202-make-namespace-with-mred))
	  (queue-callback (lambda ()
			    (current-namespace subtalk-namespace)
			    (print-struct #t)
			    (set-subtalk-ports)))))))

  (define (subtalk-eval expr)
    (parameterize ([current-eventspace subtalk-eventspace])
      (queue-callback (lambda () (eval expr)))))

  (define plain-style (make-object style-delta% 'change-normal))
  (define red-it-style (send (make-object style-delta% 'change-style 'italic)
			     set-delta-foreground
			     "red"))

  (send plain-style set-delta 'change-size 24)
  (send red-it-style set-delta 'change-size 24)

  (define (set-subtalk-ports)
    (let ([ow #f]
	  [e #f])
      (let ([mk-write (lambda (style)
			(lambda (s start end nonblock?)
			  (let-values ([(s) (substring s start end)]
				       [(display-w display-h) (get-display-size)])
			    (unless ow
			      (set! ow (make-object frame% "stdout" #f
						    575 200 10 (- display-h 200)
						    '(no-caption no-resize-border)))
			      (set! e (make-object text%))
			      (make-object editor-canvas% ow e '(hide-hscroll hide-vscroll))
			      (send ow show #t))
			    (let ([orig (send e last-position)])
			      (send e begin-edit-sequence)
			      (send e insert s orig)
			      (send e change-style style orig (send e last-position))
			      (send e end-edit-sequence)))
			  (- end start)))])
	(current-output-port (make-custom-output-port
			      #f
			      (mk-write plain-style)
			      void
			      void))
	(current-error-port (make-custom-output-port
			      #f
			      (mk-write red-it-style)
			      void
			      void)))))

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define (xtt s)
    (text s `(bold . modern) 24))
  (define (xtt* . x)
    (apply vl-append line-sep (map xtt x)))

  (define (half-page-item . s)
    (apply item (* client-w 3/4) s))

  (define brite-color "purple")
  (define dim-color "dark gray")
  (define hilite-color "black")

  (define (make-link s)
    (let ([p (t s)])
      (lift
       (colorize
	(vc-append
	 p
	 (linewidth 2 (hline (pict-width p) 1)))
	"blue")
       (- (+ 2 (pict-descent p))))))

  (define eval-link (make-link "eval"))
  
  (define (demo-slide/title s thunk . x)
    (slide/title/inset
     s
     demo-inset
     (lbl-superimpose
      (ct-superimpose
       (apply-slide-inset demo-inset titleless-page)
       (apply
	vl-append
	font-size
	x))
      (clickback eval-link thunk))))

  (define (mk-code-eval code)
    (lambda ()
      (begin-subtalk)
      (subtalk-eval (read (open-input-string 
			   (format "(begin ~a)"
				   (apply string-append code)))))))

  (define eval-code-slide
    (lambda (title . code)
      (demo-slide/title
       title
       (mk-code-eval (filter string? code))
       (apply tt*/normal code))))

  (define (eval-code-block . code)
    (hbl-append
     font-size
     (apply tt*/normal code)
     (clickback
      eval-link
      (mk-code-eval (filter string? code)))))

  (define (tt*/dimmer dimmer l)
    (apply
     vl-append
     line-sep
     (let loop ([l l])
       (cond
	[(null? l) null]
	[(or (null? (cdr l)) (string? (cadr l)))
	 (cons (dimmer (xtt (car l))) (loop (cdr l)))]
	[else
	 (cons (colorize (let loop ([s (car l)])
			   (cond 
			    [(number? (cadr l))
			     (let ([len (string-length s)]
				   [out (if (negative? (cadr l))
					    dimmer
					    values)]
				   [in (if (negative? (cadr l))
					   values
					   dimmer)]
				   [n (abs (cadr l))])
			       (hbl-append 
				(in (xtt (substring s 0 (- len n))))
				(out (xtt (substring s (- len n) (- len (sub1 n)))))
				(in (xtt (substring s (- len (sub1 n)) len)))))]
			    [(eq? (cadr l) '<<) (xtt s)]
			    [(and (pair? (cadr l)) (string? (caadr l)))
			     (let ([m (regexp-match-positions (caadr l) s)])
			       (if m
				   (hbl-append
				    (xtt (substring s 0 (caar m)))
				    (colorize (xtt (substring s (caar m) (cdar m))) brite-color)
				    (loop (substring s (cdar m) (string-length s))))
				   (xtt s)))]
			    [else (error "bad dimmer")]))
			 hilite-color)
	       (loop (cddr l)))]))))

  (define (tt*/dim . l)
    (tt*/dimmer (lambda (x) (colorize x dim-color)) l))

  (define (tt*/normal . l)
    (tt*/dimmer values l))

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (slide/center
   (vc-append
    line-sep
    (titlet "Lightweight Languages as")
    (titlet "Lightweight Operating Systems"))
   (blank)
   (bitmap (build-path (collection-path "icons") "plt.gif"))
   (blank)
   (vc-append
    line-sep
    (colorize (t "Matthew Flatt") "blue")
    (blank font-size)
    (bt "PLT")
    (t "University of Utah")))

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  (define (big s)
    (text s `(bold . ,main-font) (floor (* #e1.5 font-size))))

  (define (mk= on?)
    (slide/title/center
     "Premise"
     (big "programming language")
     (big "=")
     (big "operating system")
     (blank)
     (blank)
     ((if on? values ghost)
      (colorize (t "formal semantics for either is a virtual machine") "blue"))))
  
  (mk= #f)
  (mk= #t)

  (define windows-icon (bitmap "windows.gif"))
  (define macos-icon (bitmap "macos.gif"))
  (define linux-icon (bitmap "linux.gif"))

  (define java-icon (bitmap "java.gif"))
  (define python-icon (bitmap "python.gif"))
  (define plt-icon (bitmap "plt.gif"))

  (define (lbl-icon icon name)
    (hc-append (/ font-size 2) icon (bt name)))

  (slide/title/center
   "Operating Systems vs Programming Languages"
   (hc-append
    (* 4 font-size)
    (vc-append
     (* 4 font-size)
     (lbl-icon windows-icon "Windows")
     (inset (lbl-icon macos-icon "Mac OS") 0 0 (* 3 font-size) 0)
     (lbl-icon linux-icon "Linux"))
    
    (vc-append
     (* 4 font-size)
     (lbl-icon java-icon "Java")
     (inset (lbl-icon python-icon "Python") (* 3 font-size) 0 0 0)
     (lbl-icon plt-icon "PLT Scheme"))))

  (define-syntax (with-steps stx)
    (syntax-case stx ()
      [(_ (step-name ...) expr)
       (let ([capturing (lambda (s)
			  (datum->syntax-object #'expr s))])
	 (with-syntax ([after (capturing 'after)]
		       [vafter (capturing 'vafter)]
		       [between (capturing 'between)]
		       [between-excl (capturing 'between-excl)])
	   #'(let ([steps '(step-name ...)])
	       (map (lambda (step)
		      (define (after p)
			(memq step (or (memq p steps) null)))
		      (define (vafter p)
			(if (after p) values ghost))
		      (define (between p1 p2)
			(and (after p1) (or (eq? step p2) (not (after p2)))))
		      (define (between-excl p1 p2)
			(and (after p1) (not (after p2))))
		      expr)
		    steps))))]))

  (define implies (text "\336" 'symbol font-size))

  (define spectrum
    (let* ([bm (make-object bitmap% (* client-w 3/4) font-size)]
	   [dc (make-object bitmap-dc% bm)])
      (let ([c (make-object color%)]
	    [w (* client-w 3/4)]
	    [p (send dc get-pen)])
	(let loop ([i 0])
	  (send c set 
		(inexact->exact (floor (* 255 (/ (- w i) w))))
		0
		(inexact->exact (floor (* 255 (/ i w)))))
	  (send dc set-pen (find-pen c))
	  (send dc draw-line i 0 i font-size)
	  (unless (= i w)
	    (loop (add1 i))))
	(send dc set-pen p))
      (send dc set-bitmap #f)
      (bitmap bm)))

  (with-steps
   (ospl bias spectrum move)
   (slide/title/center
    "Operating Systems vs Programming Languages"
    (vc-append
     (rc-superimpose
      (lc-superimpose
       (colorize (hc-append font-size (bt "OS") ((vafter 'move) (arrow font-size 0))) "red")
       (blank (* client-w 3/4) (* 2 font-size)))
      (colorize (hc-append font-size ((vafter 'move) (arrow font-size (- pi))) (bt "PL")) "blue"))
     ((vafter 'spectrum) spectrum)
     (blank)
     (ht-append
      (* client-w #e.15)
      (para (* client-w #e.3)
	    "virtualize machine"
	    "for non-interference")
      (para/r (* client-w #e.3) 
	      "virtualize machine"
	      "for expressiveness"))
     (blank font-size)
     ((vafter 'bias)
      (ht-append
       (* client-w #e.15)
       (para (* client-w #e.3) implies (bt "isolation"))
       (para/r (* client-w #e.3) implies (bt "cooperation")))))))


  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define prog-process-title "Programming with Processes")

  (slide/title/center
   prog-process-title 
   (bitmap "installer.bmp"))

  '(slide/title/center
   prog-process-title   
   (bitmap "browser.bmp"))

  (slide/title/center
   prog-process-title
   (let ([ws (bitmap "web-server.gif")]
	 [ie (bitmap "ie.bmp")])
     (let ([ies (map launder (list ie ie ie ie))])
       (let ([p (vc-append
		 (* 2 (pict-height ws))
		 ws
		 (apply
		  hc-append
		  (* 2 (pict-width ie))
		  ies))])
	 (foldl (lambda (ie p)
		    (add-line p
			      ws find-cb
			      ie find-ct
			      1 dim-color))
		p ies)))))
       
   
  (slide/title/center
   prog-process-title
   (bitmap "sirmail.bmp"))

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (slide/title/center
   "Manifesto"
   (colorize (page-para "What:") "blue")
   (page-item "PL designers should think more as OS designers")
   (page-item "Maintain PL bias toward cooperation")
   (blank)
   (colorize (page-para "How:") "blue")
   (page-item "Break process abstraction into pieces")
   (page-item "Replace isolation with enforced abstraction"))

  (define outline
    (make-outline 
     'motivation "Motivation and Manifesto" null
     'plt-scheme "PLT Scheme as an Example" (lambda (sym)
					      (vl-append
					       line-sep
					       (half-page-item "Threads")
					       (half-page-item "Parameters")
					       (half-page-item "Eventspaces")
					       (half-page-item "Custodians")
					       (half-page-item "Inspectors")))
     'scheme-esq "Assembling the Pieces" (lambda (sym)
					   (half-page-item
					    (clickback
					     (make-link "SchemeEsq")
					     (lambda ()
					       (end-subtalk)
					       (begin-subtalk)
					       (subtalk-eval '(require "scheme-esq.scm"))))
					    ", a mini DrScheme"
					    (blank font-size)
					    (colorize (t "[ICFP99]") dim-color)))))

  (outline 'plt-scheme)

  (define (aspect s)
    (colorize
     (page-para* 
      ;; capitalize s:
      (it (string-append (string (char-upcase (string-ref s 0))) (substring s 1 (string-length s)))))
     "blue"))


  (slide/title/tall/inset
   "Threads"
   demo-inset
   (aspect "concurrent execution")
   (blank)
   (vl-append
    (hbl-append
     font-size
     (xtt* 
      "(require \"spin-display.scm\")")
     (clickback
      eval-link
      (lambda ()
	(end-subtalk)
	(begin-subtalk)
	(subtalk-eval '(require "spin-display.scm")))))
    (eval-code-block
     " "
     "(define (spin)"
     "  (rotate-a-little)"
     "  (sleep 0.1)"
     "  (spin))"
     " "
     "(define spinner (thread spin))" '("thread"))
    (eval-code-block
     " "
     "(kill-thread spinner)" '("kill-thread"))))
  
  (slide/title/tall/inset
   "Parameters  (a.k.a. Fluid Variables)"
   demo-inset
   (aspect "thread-local state")
   (vl-append
    line-sep
    (eval-code-block
     "(printf \"Hello\\n\")"
     "(fprintf (current-output-port) \"Hola\\n\")" '("current-.*port")
     "(fprintf (current-error-port) \"Goodbye\\n\")" '("current-.*port")
     "(error \"Ciao\")")
    (eval-code-block
     " "
     "(parameterize ((current-error-port (current-output-port)))" '("parameterize")
     "  (error \"Au Revoir\"))")
    (eval-code-block
     " "
     "(parameterize ((current-error-port (current-output-port)))"  '("parameterize")
     "  (thread"  '("thread")
     "    (lambda ()"
     "      (error \"Zai Jian\"))))")))
   
  (slide/title/tall/inset
   "Eventspaces"
   demo-inset
   (aspect "concurrent GUIs")
   (blank)
   (vl-append
    line-sep
    (eval-code-block
     "(thread (lambda () (message-box \"One\" \"Hi\")))"
     "(thread (lambda () (message-box \"Two\" \"Bye\")))")
    (eval-code-block
     " "
     " "
     "(thread (lambda () (message-box \"One\" \"Hi\")))"
     "(parameterize ((current-eventspace (make-eventspace)))" '("(make|current)-eventspace")
     "  (thread (lambda () (message-box \"Two\" \"Bye\"))))")))

  (slide/title/tall/inset
   "Custodians"
   demo-inset
   (aspect "termination and clean-up")
   (blank)
   (vl-append
    line-sep
    (hbl-append
     font-size
     (tt*/normal
      "(define c (make-custodian))" '("make-custodian")
      "(parameterize ((current-custodian c))" '("current-custodian")
      "  ...)")
     (clickback
      eval-link
      (lambda ()
	(end-subtalk)
	(begin-subtalk)
	(subtalk-eval '(define c (make-custodian)))
	(subtalk-eval '(parameterize ((current-custodian c))
			 (parameterize ((current-eventspace (make-eventspace)))
			   (dynamic-require "start-a-lot.scm" #f)))))))
    (eval-code-block
     " "
     "(custodian-shutdown-all c)" '("cust.*all"))))
   
  (slide/title/tall/inset
   "Inspectors"
   demo-inset
   (aspect "debugging access to data")
   (blank)
   (vl-append
    line-sep
    (eval-code-block
     "(define-struct fish (color weight))"
     "(define eddie (make-fish 'red 100))"
     "(print eddie)")
    (eval-code-block
     " "
     " "
     "(define senior-inspector (current-inspector))" '("(senior-insepctor)|(current-inspector)")
     "(parameterize ([current-inspector (make-inspector)])"  '("make-inspector")
     "  (define-struct fish (color weight))"
     "  (define eddie (make-fish 'red 100))"
     "  (parameterize ([current-inspector senior-inspector])"  '("senior-inspector")
     "    (print eddie)))")))

  (slide/title
   "Etc."
   (half-page-item "Security Guards")
   (aspect "resource access control")
   (half-page-item "Namespaces")
   (aspect "global bindings")
   (half-page-item "Will Executors")
   (aspect "timing of finalizations"))

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (outline 'scheme-esq)

  (demo-slide/title
   "GUI - Frame"
   (lambda ()
     (end-subtalk)
     (begin-subtalk)
     (subtalk-eval 
      '(begin 
	 (define frame (instantiate frame% () [label "SchemeEsq"] [width 400] [height 185]))
	 (send frame show #t))))
   (tt*/normal
    "(define frame"
    "  (instantiate frame% () " '("frame%")
    "    [label \"SchemeEsq\"]"
    "    [width 400] [height 175]))"
    ""
    "(send frame show #t)"))

  (eval-code-slide
   "GUI - Reset Button"
   "(instantiate button% ()" '("button%")
   "  [label \"Reset\"]"
   "  [parent frame]"
   "  [callback (lambda (b e) (reset-program))])")

  (eval-code-slide
   "GUI - Interaction Area"
   "(define repl-display-canvas"
   "  (instantiate editor-canvas% ()" '("editor-canvas%")
   "    [parent frame]))")

  (demo-slide/title
   "GUI - Interaction Buffer"
   (lambda ()
     (subtalk-eval '(begin
		      (load "text.ss")
		      (define repl-editor (make-object esq-text%))
		      (send repl-display-canvas set-editor repl-editor))))
   (tt*/normal
    "(define esq-text% "
    "  (class text% ... (evaluate str) ...))" '("text%")
    " "
    "(define repl-editor (instantiate esq-text% ()))"
    "(send repl-display-canvas set-editor repl-editor)"))

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (eval-code-slide
   "Evaluator"
   "(define (evaluate expr-str)"
   "  (thread" '("thread")
   "    (lambda ()"
   "      (print (eval (read (open-input-string expr-str))))" '("(read)|(eval)|(print)")
   "      (newline)"
   "      (send repl-editor new-prompt))))")

  (demo-slide/title
   "Evaluator Output"
   (lambda ()
     (subtalk-eval
      '(begin
	 (define user-output-port
	   (make-custom-output-port
	    #f
	    (lambda (s start end nonblock?) 
	      (send repl-editor output (substring s start end))
	      (- end start))
	    void void))

	 (define (evaluate expr-str)
	   (thread
	    (lambda ()
	      (current-output-port user-output-port)
	      (with-handlers ((exn? 
			       (lambda (exn)
				 (display (exn-message exn)))))
		(print (eval (read (open-input-string expr-str)))))
	      (newline)
	      (send repl-editor new-prompt)))))))
   (tt*/dim
      "(define user-output-port" '<<
      "  (make-custom-output-port ... repl-editor ...))" '<<
      " "
      "(define (evaluate expr-str)"
      "  (parameterize ((current-output-port user-output-port))" '<<
      "    (thread"
      "      (lambda ()"
      "        ...))))" '+2))

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (demo-slide/title
   "Evaluating GUIs"
   (lambda ()
     (subtalk-eval
      '(begin
	 (define user-eventspace (make-eventspace))
	 
	 (define (evaluate expr-str)
	   (thread
	    (lambda ()
	      (current-output-port user-output-port)
	      (current-eventspace user-eventspace)
	      (with-handlers ((exn? 
			       (lambda (exn)
				 (display (exn-message exn)))))
		(print (eval (read (open-input-string expr-str)))))
	      (newline)
	      (send repl-editor new-prompt)))))))
   (vl-append
    (tt*/dim
     "(define user-eventspace (make-eventspace))" '<<
     " "
     "(define (evaluate expr-str)"
     "  (parameterize ((current-output-port user-output-port)"
     "                 (current-eventspace user-eventspace))" -1
     "    (thread"
     "      (lambda ()"
     "        ...)))")))

  (demo-slide/title
   "Custodian for Evaluation"
   (lambda ()
     (subtalk-eval
      '(begin
	 (define user-custodian (make-custodian))
	 (define user-eventspace
	   (parameterize ((current-custodian user-custodian))
	     (make-eventspace)))
	 (define (evaluate expr-str)
	   (parameterize ((current-custodian user-custodian))
	     (thread
	      (lambda ()
		(current-output-port user-output-port)
		(current-eventspace user-eventspace)
		(with-handlers ((exn? 
				 (lambda (exn)
				   (display (exn-message exn)))))
		  (print (eval (read (open-input-string expr-str)))))
		(newline)
		(send repl-editor new-prompt))))))))
   (tt*/dim
    "(define user-custodian (make-custodian))" '<<
    " "
    "(define user-eventspace" '<<
    "  (parameterize ((current-custodian user-custodian))" '<<
    "    (make-eventspace)))" '<<
    " "
    "(define (evaluate expr-str)" 
    "  (parameterize ((current-output-port user-output-port)"
    "                 (current-eventspace user-eventspace)"
    "                 (current-custodian user-custodian))" -1
    "    (thread"
    "      (lambda ()"
    "        ...))))"))

  (eval-code-slide
   "Reset Evaluation"
   "(define (reset-program)"
   "  (custodian-shutdown-all user-custodian)"
   " "
   "  (parameterize ((current-custodian user-custodian))"
   "    (set! user-eventspace (make-eventspace)))"
   "  (send repl-editor reset))")

  (demo-slide/title
   "Inspecting Results"
   (lambda ()
     (subtalk-eval
      '(begin
	 (define esq-inspector (current-inspector))
	 (define user-inspector (make-inspector))
	 (define (evaluate expr-str)
	   (parameterize ((current-custodian user-custodian)
			  (current-inspector user-inspector))
	     (thread
	      (lambda ()
		(current-output-port user-output-port)
		(current-eventspace user-eventspace)
		(with-handlers ((exn? 
				 (lambda (exn)
				   (display (exn-message exn)))))
		  (let ([v (eval (read (open-input-string expr-str)))])
		    (parameterize ((current-inspector esq-inspector))
		      (print v))))
		(newline)
		(send repl-editor new-prompt))))))))
   (tt*/dim
    "(define esq-inspector (current-inspector))" '<<
    "(define user-inspector (make-inspector))" '<<
    " "
    "(define (evaluate expr-str)" 
    "  (parameterize (..."
    "                 (current-inspector user-inspector))" -1
    "    (thread"
    "      (lambda ()"
    "        (let ([v (eval ...)])" '<<
    "          (parameterize ((current-inspector esq-inspector))" '<<
    "            (print v))" '<<
    "        ...))))"))

  ;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (slide/title
   "Conclusion"
   (page-item "Programmers need OS-like constructs in languages")
   (page-subitem "run-time environment (e.g., script GUI wrapper, talk demos)")
   (page-subitem "easy termination (e.g., server sessions," 
		 (clickback (make-link "awry demos")
			    (lambda ()
			      (end-subtalk)
			      (begin-subtalk)
			      (subtalk-eval '(require "scheme-esq.scm"))))
		 ")")
   (page-subitem "concurrency (e.g., browser cancel, mail client)")
   (page-item "Multiple language constructs for \"process\"")
   (page-subitem (vl-append
		  line-sep
		  (t "programmer can mix and match to")
		  (t "balance isolation and cooperation"))))
  
  'done)
